package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/md"
	"encoding/json"
	"errors"
	"github.com/shopspring/decimal"
	"xorm.io/xorm"
)

// CalcPriceIncreaseFormula 计算涨价公式(【用户资金 ÷（用户资金+原始资金）÷ 原始数量】 = 用户获得绿色能量个数)
func CalcPriceIncreaseFormula(engine *xorm.Engine, userAmountValue string) (err error, values, nowPriceValue, afterPriceValue string) {
	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	userAmount, _ := decimal.NewFromString(userAmountValue)                                                  //用户资金
	nowPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)                         //当前价格
	originalQuantityNums, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalQuantityNums) //原始数量
	originalQuantityFunds, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalFunds)       //原始资金
	afterPrice := (userAmount.Add(originalQuantityFunds)).Div(originalQuantityNums).Truncate(8)              //涨价后的价格

	values = userAmount.Div(afterPrice).Truncate(8).String() //用户得到绿色能量个数
	nowPriceValue = nowPrice.String()
	afterPriceValue = afterPrice.String()
	return
}

// NewCalcPriceIncreaseFormula 计算涨价公式(【用户资金 ÷（用户资金+原始资金）÷ 原始数量】 = 用户获得绿色能量个数)
func NewCalcPriceIncreaseFormula(userAmountValue string, oneCirclesGreenEnergyBasicSetting *model.OneCirclesGreenEnergyBasicSetting) (err error, values, nowPriceValue, afterPriceValue string) {
	userAmount, _ := decimal.NewFromString(userAmountValue)                                                  //用户资金
	nowPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)                         //当前价格
	originalQuantityNums, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalQuantityNums) //原始数量
	originalQuantityFunds, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalFunds)       //原始资金
	afterPrice := (userAmount.Add(originalQuantityFunds)).Div(originalQuantityNums).Truncate(8)              //涨价后的价格

	values = userAmount.Div(afterPrice).Truncate(8).String() //用户得到绿色能量个数
	nowPriceValue = nowPrice.String()
	afterPriceValue = afterPrice.String()
	return
}

// CalcPriceReductionFormula 计算降价公式(【用户需兑换绿能色量数量*｛原始资金÷（用户需兑换绿色数量+原始数量｝*（1 - 扣比例50% ~ 23%） = 用户获得绿色能量个数)
func CalcPriceReductionFormula(engine *xorm.Engine, userExchangeNumsValue string, levelId string) (err error, values, greenEnergy, greenEnergyFee, nowPriceValue, afterPriceValue string) {
	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	var vipEquitySetting []*md2.VipEquitySettingStruct
	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.VipEquitySetting), &vipEquitySetting)
	if err != nil {
		return
	}
	var exchangeAccountBalanceFeeValue string
	for _, v := range vipEquitySetting {
		if v.VipLevelId == levelId {
			exchangeAccountBalanceFeeValue = v.ExchangeAccountBalanceFee
		}
	}
	if exchangeAccountBalanceFeeValue == "" {
		err = errors.New("未查询到当前会员等级兑换余额手续费")
		return
	}
	decimalRate := decimal.NewFromInt(100)                                                                   //百分比
	exchangeAccountBalanceFee, _ := decimal.NewFromString(exchangeAccountBalanceFeeValue)                    //兑换手续费
	userExchangeNums, _ := decimal.NewFromString(userExchangeNumsValue)                                      //用户兑换绿色能量
	originalQuantityNums, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalQuantityNums) //原始数量
	nowPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)                         //当前价格
	originalQuantityFunds, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalFunds)       //原始资金
	afterPrice := originalQuantityFunds.Div(userExchangeNums.Add(originalQuantityNums))                      //降价后的价格

	greenEnergyValues := userExchangeNums.Mul(afterPrice)                                     //绿色能量个数
	greenEnergyFeeValues := greenEnergyValues.Mul(exchangeAccountBalanceFee.Div(decimalRate)) //绿色能量个数扣除手续费价值

	values = greenEnergyValues.Sub(greenEnergyFeeValues).Truncate(8).String()                              //用户实际得到的钱
	greenEnergy = greenEnergyValues.Truncate(8).String()                                                   //原本兑换到的钱
	greenEnergyFee = userExchangeNums.Mul(exchangeAccountBalanceFee.Div(decimalRate)).Truncate(8).String() //绿色能量手续费
	nowPriceValue = nowPrice.String()
	afterPriceValue = afterPrice.String()
	return
}
